From 69e947c13a33a60726134f62178146f8b3a3bc67 Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Benedek=20L=C3=A1szl=C3=B3?= <laszlo.benedek@hu.digi.tv>
Date: Thu, 22 Jul 2010 13:50:37 +0200
Subject: [PATCH 1/2] Role support

Rolename parameter can be specified in each server's saved configuration. When specified,
issue a SET role=<rolename> command after a successfull connection to the server.
---
 pgadmin/db/pgConn.cpp                     |   17 +-
 pgadmin/dlg/dlgSelectConnection.cpp       |   13 +-
 pgadmin/dlg/dlgServer.cpp                 |    9 +-
 pgadmin/frm/frmMain.cpp                   |    1 +
 pgadmin/frm/frmStatus.cpp                 |    6 +-
 pgadmin/include/db/pgConn.h               |    7 +-
 pgadmin/include/dlg/dlgSelectConnection.h |    2 +-
 pgadmin/include/schema/pgServer.h         |    6 +-
 pgadmin/pgAdmin3.cpp                      |   12 +-
 pgadmin/schema/pgRole.cpp                 |    1 +
 pgadmin/schema/pgServer.cpp               |   20 +-
 pgadmin/ui/dlgSelectConnection.xrc        |    8 +
 pgadmin/ui/dlgServer.xrc                  |   16 +-
 pgadmin/ui/xrcDialogs.cpp                 |  642 +++++++++++++++--------------
 pgadmin/utils/sysSettings.cpp             |    1 +
 15 files changed, 430 insertions(+), 331 deletions(-)

diff --git a/pgadmin/db/pgConn.cpp b/pgadmin/db/pgConn.cpp
index 353e487..e876c65 100644
--- a/pgadmin/db/pgConn.cpp
+++ b/pgadmin/db/pgConn.cpp
@@ -50,7 +50,7 @@ static void pgNoticeProcessor(void *arg, const char *message)
     ((pgConn*)arg)->Notice(message);
 }
 
-pgConn::pgConn(const wxString& server, const wxString& database, const wxString& username, const wxString& password, int port, int sslmode, OID oid, const wxString& applicationname)
+pgConn::pgConn(const wxString& server, const wxString& database, const wxString& username, const wxString& password, int port, const wxString& rolename, int sslmode, OID oid, const wxString& applicationname)
 {
     wxString msg;
 
@@ -59,6 +59,7 @@ pgConn::pgConn(const wxString& server, const wxString& database, const wxString&
     save_username = username;
     save_password = password;
     save_port = port;
+    save_rolename = rolename;
     save_sslmode = sslmode;
     save_oid = oid;
     save_applicationname = applicationname;
@@ -123,6 +124,7 @@ pgConn::pgConn(const wxString& server, const wxString& database, const wxString&
 	
     dbHost = server;
     dbHostName = server;
+    dbRole = rolename;
 
     if (!applicationname.IsEmpty())
     {
@@ -176,7 +178,16 @@ bool pgConn::DoConnect()
         connStatus = PGCONN_OK;
         PQsetNoticeProcessor(conn, pgNoticeProcessor, this);
 
-        wxString sql=wxT("SET DateStyle=ISO;SELECT oid, pg_encoding_to_char(encoding) AS encoding, datlastsysoid\n")
+        wxString sql;
+        if ( dbRole != wxEmptyString) {
+            sql += wxT("SET Role=");
+            wxString role = dbRole;
+            role.Replace(wxT("\\"), wxT("\\\\"));
+            role.Replace(wxT("'"), wxT("''"));
+            sql += qtString(role) + wxT(";");
+        }
+
+        sql += wxT("SET DateStyle=ISO;SELECT oid, pg_encoding_to_char(encoding) AS encoding, datlastsysoid\n")
                       wxT("  FROM pg_database WHERE ");
 
         if (save_oid)
@@ -256,7 +267,7 @@ bool pgConn::Reconnect()
 
 pgConn *pgConn::Duplicate()
 {
-    return new pgConn(wxString(save_server), wxString(save_database), wxString(save_username), wxString(save_password), save_port, save_sslmode, save_oid);
+    return new pgConn(wxString(save_server), wxString(save_database), wxString(save_username), wxString(save_password), save_port, save_rolename, save_sslmode, save_oid);
 }
 
 
diff --git a/pgadmin/dlg/dlgSelectConnection.cpp b/pgadmin/dlg/dlgSelectConnection.cpp
index fcd388b..7f2a63d 100644
--- a/pgadmin/dlg/dlgSelectConnection.cpp
+++ b/pgadmin/dlg/dlgSelectConnection.cpp
@@ -32,12 +32,14 @@ BEGIN_EVENT_TABLE(dlgSelectConnection, DialogWithHelp)
 	EVT_TEXT(CTRLID_CBSERVER,          dlgSelectConnection::OnTextChange)
 	EVT_TEXT(CTRLID_CBDATABASE,        dlgSelectConnection::OnTextChange)
 	EVT_TEXT(XRCID("cbUsername"),      dlgSelectConnection::OnTextChange)
+	EVT_TEXT(XRCID("txtRolename"),     dlgSelectConnection::OnTextChange)
     EVT_BUTTON (wxID_OK,               dlgSelectConnection::OnOK)
     EVT_BUTTON (wxID_CANCEL,           dlgSelectConnection::OnCancel)
 END_EVENT_TABLE()
 
 #define stUsername		CTRL_STATIC("stUsername")
 #define cbUsername		CTRL_COMBOBOX("cbUsername")
+#define txtRolename		CTRL_TEXT("txtRolename")
 
 
 
@@ -60,6 +62,7 @@ DialogWithHelp(form)
 		cbServer->SetValue(settings->Read(wxT("QuickConnect/server"), wxEmptyString));
 		cbDatabase->SetValue(settings->Read(wxT("QuickConnect/database"), wxEmptyString));
 		cbUsername->SetValue(settings->Read(wxT("QuickConnect/username"), wxEmptyString));
+		txtRolename->SetValue(settings->Read(wxT("QuickConnect/rolename"), wxEmptyString));
 	}
 
 	btnOK->Enable(cbServer->GetValue().Length() > 0 && cbDatabase->GetValue().Length() > 0 && cbUsername->GetValue().Length() > 0);
@@ -190,6 +193,7 @@ pgConn *dlgSelectConnection::CreateConn(wxString& applicationname, bool& created
 	}
 
     wxString user = cbUsername->GetValue();
+    wxString role = txtRolename->GetValue();
 
     if (cbConnection)
     {
@@ -201,6 +205,7 @@ pgConn *dlgSelectConnection::CreateConn(wxString& applicationname, bool& created
                 conn->GetHost() == serv &&
                 conn->GetPort() == port &&
                 conn->GetUser() == user &&
+                conn->GetRole() == role &&
                 conn->GetDbname() == db)
             {
                 createdNew = false;
@@ -212,13 +217,13 @@ pgConn *dlgSelectConnection::CreateConn(wxString& applicationname, bool& created
     
     int sslmode = remoteServer ? remoteServer->GetSSL() : 0;
 
-	return CreateConn(serv, db, user, port, sslmode, applicationname, true);
+	return CreateConn(serv, db, user, port, role, sslmode, applicationname, true);
 }
 
-pgConn *dlgSelectConnection::CreateConn(wxString& server, wxString& dbname, wxString& username, int port, int sslmode, wxString& applicationname, bool writeMRU)
+pgConn *dlgSelectConnection::CreateConn(wxString& server, wxString& dbname, wxString& username, int port, wxString& rolename, int sslmode, wxString& applicationname, bool writeMRU)
 {
 	pgConn *newconn;
-	newconn = new pgConn(server, dbname, username, wxT(""), port, sslmode, 0, applicationname);
+	newconn = new pgConn(server, dbname, username, wxT(""), port, rolename, sslmode, 0, applicationname);
 	if (newconn->GetStatus() != PGCONN_OK &&
 		newconn->GetLastError().Cmp(wxString(PQnoPasswordSupplied, wxConvUTF8)) == 0)
 	{
@@ -232,7 +237,7 @@ pgConn *dlgSelectConnection::CreateConn(wxString& server, wxString& dbname, wxSt
 		if (dlg.Go() != wxID_OK)
 			return NULL;
 
-		newconn = new pgConn(server, dbname, username, dlg.GetPassword(), port, sslmode, 0, applicationname);
+		newconn = new pgConn(server, dbname, username, dlg.GetPassword(), port, rolename, sslmode, 0, applicationname);
 	}
 
 	if (newconn)
diff --git a/pgadmin/dlg/dlgServer.cpp b/pgadmin/dlg/dlgServer.cpp
index 011ed78..2a537cd 100644
--- a/pgadmin/dlg/dlgServer.cpp
+++ b/pgadmin/dlg/dlgServer.cpp
@@ -37,6 +37,7 @@
 #define chkTryConnect   CTRL_CHECKBOX("chkTryConnect")
 #define stStorePwd      CTRL_STATIC("stStorePwd")
 #define chkStorePwd     CTRL_CHECKBOX("chkStorePwd")
+#define txtRolename     CTRL_TEXT("txtRolename")
 #define stRestore       CTRL_STATIC("stRestore")
 #define chkRestore      CTRL_CHECKBOX("chkRestore")
 #define stPassword      CTRL_STATIC("stPassword")
@@ -54,6 +55,7 @@ BEGIN_EVENT_TABLE(dlgServer, dlgProperty)
     EVT_COMBOBOX(XRCID("cbDatabase"),               dlgProperty::OnChange)
     EVT_TEXT(XRCID("txtPort")  ,                    dlgProperty::OnChange)
     EVT_TEXT(XRCID("txtUsername"),                  dlgProperty::OnChange)
+    EVT_TEXT(XRCID("txtRolename"),                  dlgProperty::OnChange)
     EVT_TEXT(XRCID("txtDbRestriction"),             dlgServer::OnChangeRestr)
     EVT_COMBOBOX(XRCID("cbSSL"),                    dlgProperty::OnChange)
     EVT_CHECKBOX(XRCID("chkStorePwd"),              dlgProperty::OnChange)
@@ -170,6 +172,7 @@ void dlgServer::OnOK(wxCommandEvent &ev)
         server->iSetSSL(cbSSL->GetCurrentSelection());
         server->iSetDatabase(cbDatabase->GetValue());
         server->iSetUsername(txtUsername->GetValue());
+        server->iSetRolename(txtRolename->GetValue());
         server->iSetStorePwd(chkStorePwd->GetValue());
         server->iSetRestore(chkRestore->GetValue());
         server->iSetDbRestriction(txtDbRestriction->GetValue().Trim());
@@ -195,6 +198,7 @@ void dlgServer::OnOK(wxCommandEvent &ev)
                 server->GetUsername(), 
                 server->GetPort(), 
                 server->GetStorePwd(),
+                server->GetRolename(),
                 server->GetRestore(),
                 server->GetSSL(),
 	        	server->GetColour(),
@@ -354,6 +358,7 @@ int dlgServer::Go(bool modal)
         cbDatabase->SetValue(server->GetDatabaseName());
         txtUsername->SetValue(server->GetUsername());
         chkStorePwd->SetValue(server->GetStorePwd());
+        txtRolename->SetValue(server->GetRolename());
         chkRestore->SetValue(server->GetRestore());
         txtDbRestriction->SetValue(server->GetDbRestriction());
         colourPicker->SetColour(server->GetColour());
@@ -369,6 +374,7 @@ int dlgServer::Go(bool modal)
             cbSSL->Disable();
             txtUsername->Disable();
             chkStorePwd->Disable();
+            txtRolename->Disable();
         }
     }
     else
@@ -402,7 +408,7 @@ pgObject *dlgServer::CreateObject(pgCollection *collection)
     pgServer *obj=new pgServer(GetName(), txtDescription->GetValue(), cbDatabase->GetValue(), 
         txtUsername->GetValue(), StrToLong(txtPort->GetValue()), 
         chkTryConnect->GetValue() && chkStorePwd->GetValue(), 
-        chkRestore->GetValue(), cbSSL->GetCurrentSelection(),
+        txtRolename->GetValue(), chkRestore->GetValue(), cbSSL->GetCurrentSelection(),
 		colourPicker->GetColourString(), cbGroup->GetValue());
 
     obj->iSetDbRestriction(txtDbRestriction->GetValue().Trim());
@@ -444,6 +450,7 @@ void dlgServer::CheckChange()
                || txtUsername->GetValue() != server->GetUsername()
                || cbSSL->GetCurrentSelection() != server->GetSSL()
                || chkStorePwd->GetValue() != server->GetStorePwd()
+               || txtRolename->GetValue() != server->GetRolename()
                || chkRestore->GetValue() != server->GetRestore()
                || txtDbRestriction->GetValue() != server->GetDbRestriction()
                || sColour != sColour2
diff --git a/pgadmin/frm/frmMain.cpp b/pgadmin/frm/frmMain.cpp
index f2ad845..9512bff 100644
--- a/pgadmin/frm/frmMain.cpp
+++ b/pgadmin/frm/frmMain.cpp
@@ -1163,6 +1163,7 @@ void frmMain::StoreServers()
                     settings->Write(key + wxT("DiscoveryID"), server->GetDiscoveryID());
                     settings->Write(key + wxT("Port"), server->GetPort());
                     settings->Write(key + wxT("StorePwd"), server->GetStorePwd());
+                    settings->Write(key + wxT("Rolename"), server->GetRolename());
                     settings->Write(key + wxT("Restore"), server->GetRestore());
                     settings->Write(key + wxT("Database"), server->GetDatabaseName());
                     settings->Write(key + wxT("Username"), server->GetUsername());
diff --git a/pgadmin/frm/frmStatus.cpp b/pgadmin/frm/frmStatus.cpp
index a1881b3..928a968 100644
--- a/pgadmin/frm/frmStatus.cpp
+++ b/pgadmin/frm/frmStatus.cpp
@@ -453,7 +453,7 @@ void frmStatus::OnChangeDatabase(wxCommandEvent &ev)
     }
 
     locks_connection = new pgConn(connection->GetHostName(), cbDatabase->GetValue(),
-      connection->GetUser(), connection->GetPassword(), connection->GetPort(), connection->GetSslMode(),
+      connection->GetUser(), connection->GetPassword(), connection->GetPort(), connection->GetRole(), connection->GetSslMode(),
       0, connection->GetApplicationName());
 
     locks_connection->ExecuteVoid(wxT("SET log_statement='none';SET log_duration='off';SET log_min_duration_statement=-1;"),false);
@@ -873,7 +873,7 @@ void frmStatus::OnCopyQuery(wxCommandEvent& ev)
     {
         pgConn *conn = new pgConn(connection->GetHostName(), dbname,
           connection->GetUser(), connection->GetPassword(),
-          connection->GetPort(), connection->GetSslMode(), connection->GetDbOid(),
+          connection->GetPort(), connection->GetRole(), connection->GetSslMode(), connection->GetDbOid(),
           connection->GetApplicationName());
         if (conn)
         {
@@ -2359,6 +2359,7 @@ void frmStatus::OnCommit(wxCommandEvent &event)
                                        connection->GetUser(), 
                                        connection->GetPassword(), 
                                        connection->GetPort(), 
+                                       connection->GetRole(),
                                        connection->GetSslMode(),
                                        0,
                                        connection->GetApplicationName());
@@ -2409,6 +2410,7 @@ void frmStatus::OnRollback(wxCommandEvent &event)
                                        connection->GetUser(), 
                                        connection->GetPassword(), 
                                        connection->GetPort(), 
+                                       connection->GetRole(),
                                        connection->GetSslMode(),
                                        0,
                                        connection->GetApplicationName());
diff --git a/pgadmin/include/db/pgConn.h b/pgadmin/include/db/pgConn.h
index cc029e6..ae4d9d5 100644
--- a/pgadmin/include/db/pgConn.h
+++ b/pgadmin/include/db/pgConn.h
@@ -81,7 +81,7 @@ typedef struct pgError {
 class pgConn
 {
 public:
-    pgConn(const wxString& server = wxT(""), const wxString& database = wxT(""), const wxString& username = wxT(""), const wxString& password = wxT(""), int port = 5432, int sslmode=0, OID oid=0, const wxString& applicationname = wxT("pgAdmin"));
+    pgConn(const wxString& server = wxT(""), const wxString& database = wxT(""), const wxString& username = wxT(""), const wxString& password = wxT(""), int port = 5432, const wxString& rolename = wxT(""), int sslmode=0, OID oid=0, const wxString& applicationname = wxT("pgAdmin"));
     ~pgConn();
 
     bool HasPrivilege(const wxString &objTyp, const wxString &objName, const wxString &priv);
@@ -110,6 +110,7 @@ public:
     pgSet *ExecuteSet(const wxString& sql);
     wxString GetUser() const { return wxString(PQuser(conn), *conv); }
     wxString GetPassword() const { return wxString(PQpass(conn), *conv); }
+    wxString GetRole() const { return dbRole; }
     wxString GetHost() const { return dbHost; }
     wxString GetHostName() const { return dbHostName; }
     wxString GetDbname() const { return save_database; }
@@ -154,7 +155,7 @@ protected:
 
     wxMBConv *conv;
     bool needColQuoting, utfConnectString;
-    wxString dbHost, dbHostName;
+    wxString dbRole, dbHost, dbHostName;
     OID lastSystemOID;
     OID dbOid;
 
@@ -177,7 +178,7 @@ private:
     wxString reservedNamespaces;
 	wxString connstr;
 	
-    wxString save_server, save_database, save_username, save_password, save_applicationname;
+    wxString save_server, save_database, save_username, save_password, save_rolename, save_applicationname;
     int save_port, save_sslmode;
     OID save_oid;
 };
diff --git a/pgadmin/include/dlg/dlgSelectConnection.h b/pgadmin/include/dlg/dlgSelectConnection.h
index 7f30be3..cdbfa35 100644
--- a/pgadmin/include/dlg/dlgSelectConnection.h
+++ b/pgadmin/include/dlg/dlgSelectConnection.h
@@ -25,7 +25,7 @@ public:
     wxString GetHelpPage() const;
     pgServer *GetServer() { return remoteServer; }
 	pgConn *CreateConn(wxString& applicationame, bool& createdNew);
-	pgConn *CreateConn(wxString& server, wxString& dbname, wxString& username, int port, int sslmode, wxString& applicationame, bool writeMRU = false);
+	pgConn *CreateConn(wxString& server, wxString& dbname, wxString& username, int port, wxString& rolename, int sslmode, wxString& applicationame, bool writeMRU = false);
 	wxString GetServerName();
     wxString GetDatabase();
 
diff --git a/pgadmin/include/schema/pgServer.h b/pgadmin/include/schema/pgServer.h
index 62f819c..fd881f6 100644
--- a/pgadmin/include/schema/pgServer.h
+++ b/pgadmin/include/schema/pgServer.h
@@ -38,7 +38,7 @@ extern pgServerFactory serverFactory;
 class pgServer : public pgObject
 {
 public:
-	pgServer(const wxString& newServer = wxT(""), const wxString& newDescription = wxT(""), const wxString& newDatabase = wxT(""), const wxString& newUsername = wxT(""), int newPort = 5432, bool storePwd=false, bool restore=true, int sslMode=0, const wxString &colour = wxT("#FFFFFF"), const wxString &group = wxEmptyString);
+	pgServer(const wxString& newServer = wxT(""), const wxString& newDescription = wxT(""), const wxString& newDatabase = wxT(""), const wxString& newUsername = wxT(""), int newPort = 5432, bool storePwd=false, const wxString& newRolename = wxT(""), bool restore=true, int sslMode=0, const wxString &colour = wxT("#FFFFFF"), const wxString &group = wxEmptyString);
     ~pgServer();
     int GetIconId();
 
@@ -65,6 +65,7 @@ public:
     wxString GetUsername() const { return username; }
     wxString GetPassword() const { return (password == wxEmptyString ? conn->GetPassword() : password); }
     bool GetStorePwd() const { return storePwd; }
+    wxString GetRolename() const { return rolename; }
     bool GetRestore() const { return restore; }
     wxString GetLastError() const;
 
@@ -116,6 +117,7 @@ public:
     void iSetUsername(const wxString& newVal) { username = newVal; }
     void iSetPassword(const wxString& newVal) { password = newVal; }
     void iSetStorePwd(const bool b) { storePwd = b; }
+    void iSetRolename(const wxString& newVal) { rolename = newVal; }
     void iSetRestore(const bool b) { restore = b; }
     bool SetPassword(const wxString& newVal);
     wxDateTime GetUpSince() { return upSince; }
@@ -156,7 +158,7 @@ private:
     pgConn *conn;
     long serverIndex;
     bool connected, passwordValid, autovacuumRunning;
-    wxString database, username, password, ver, error;
+    wxString database, username, password, rolename, ver, error;
     wxString lastDatabase, lastSchema, description, serviceId, discoveryId;
     wxDateTime upSince;
     int port, ssl;
diff --git a/pgadmin/pgAdmin3.cpp b/pgadmin/pgAdmin3.cpp
index 1f60cd5..ef54223 100644
--- a/pgadmin/pgAdmin3.cpp
+++ b/pgadmin/pgAdmin3.cpp
@@ -454,7 +454,7 @@ bool pgAdmin3::OnInit()
             else if (cmdParser.Found(wxT("Sc"), &connstr))
             {
                 wxLogInfo(wxT("Starting in server status connect mode (-Sc)."), configFile.c_str());
-                wxString host, database, username, tmps;
+                wxString host, database, username, rolename, tmps;
                 int sslmode=0,port=0;
                 wxStringTokenizer tkn(connstr, wxT(" "), wxTOKEN_STRTOK);
                 while (tkn.HasMoreTokens())
@@ -468,6 +468,8 @@ bool pgAdmin3::OnInit()
                         continue;
                     if (str.StartsWith(wxT("user="), &username))
                         continue;
+                    if (str.StartsWith(wxT("role="), &rolename))
+                        continue;
                     if (str.StartsWith(wxT("port="), &tmps))
                     {
                         port = StrToLong(tmps);
@@ -500,7 +502,7 @@ bool pgAdmin3::OnInit()
                 winSplash->Show(false);
                 dlgSelectConnection dlg(NULL, NULL);
                 dlg.CenterOnParent();
-                conn = dlg.CreateConn(host, database, username, port, sslmode, applicationname);
+                conn = dlg.CreateConn(host, database, username, port, rolename, sslmode, applicationname);
             }
             else
             {
@@ -546,7 +548,7 @@ bool pgAdmin3::OnInit()
             else if (cmdParser.Found(wxT("qc"), &connstr))
             {
                 wxLogInfo(wxT("Starting in query tool connect mode (-qc)."), configFile.c_str());
-                wxString host, database, username, tmps;
+                wxString host, database, username, rolename, tmps;
                 int sslmode=0,port=0;
                 wxStringTokenizer tkn(connstr, wxT(" "), wxTOKEN_STRTOK);
                 while (tkn.HasMoreTokens())
@@ -560,6 +562,8 @@ bool pgAdmin3::OnInit()
                         continue;
                     if (str.StartsWith(wxT("user="), &username))
                         continue;
+                    if (str.StartsWith(wxT("role="), &rolename))
+                        continue;
                     if (str.StartsWith(wxT("port="), &tmps))
                     {
                         port = StrToLong(tmps);
@@ -592,7 +596,7 @@ bool pgAdmin3::OnInit()
                 winSplash->Show(false);
                 dlgSelectConnection dlg(NULL, NULL);
                 dlg.CenterOnParent();
-                conn = dlg.CreateConn(host, database, username, port, sslmode, applicationname);
+                conn = dlg.CreateConn(host, database, username, port, rolename, sslmode, applicationname);
             }
             else
             {
diff --git a/pgadmin/schema/pgRole.cpp b/pgadmin/schema/pgRole.cpp
index b2cdb9d..ea904df 100644
--- a/pgadmin/schema/pgRole.cpp
+++ b/pgadmin/schema/pgRole.cpp
@@ -305,6 +305,7 @@ void pgRole::ReassignDropOwnedTo(frmMain *form)
                      GetConnection()->GetUser(),
                      GetConnection()->GetPassword(),
                      GetConnection()->GetPort(),
+                     GetConnection()->GetRole(),
                      GetConnection()->GetSslMode(),
                      0,
                      GetConnection()->GetApplicationName());
diff --git a/pgadmin/schema/pgServer.cpp b/pgadmin/schema/pgServer.cpp
index a201476..758e918 100644
--- a/pgadmin/schema/pgServer.cpp
+++ b/pgadmin/schema/pgServer.cpp
@@ -40,7 +40,7 @@
 
 #define DEFAULT_PG_DATABASE wxT("postgres")
 
-pgServer::pgServer(const wxString& newName, const wxString& newDescription, const wxString& newDatabase, const wxString& newUsername, int newPort, bool _storePwd, bool _restore, int _ssl, const wxString &_colour, const wxString &_group)
+pgServer::pgServer(const wxString& newName, const wxString& newDescription, const wxString& newDatabase, const wxString& newUsername, int newPort, bool _storePwd, const wxString& newRolename, bool _restore, int _ssl, const wxString &_colour, const wxString &_group)
 : pgObject(serverFactory, newName)
 {  
     description = newDescription;
@@ -59,6 +59,7 @@ pgServer::pgServer(const wxString& newName, const wxString& newDescription, cons
     conn = NULL;
     passwordValid=true;
     storePwd=_storePwd;
+    rolename = newRolename;
     restore=_restore;
     superUser=false;
     createPrivilege=false;
@@ -140,7 +141,7 @@ pgConn *pgServer::CreateConn(wxString dbName, OID oid, wxString applicationname)
         dbName = GetDatabaseName();
         oid = dbOid;
     }
-    pgConn *conn=new pgConn(GetName(), dbName, username, password, port, ssl, oid, applicationname);
+    pgConn *conn=new pgConn(GetName(), dbName, username, password, port, rolename, ssl, oid, applicationname);
 
     if (conn && conn->GetStatus() != PGCONN_OK)
     {
@@ -617,21 +618,21 @@ int pgServer::Connect(frmMain *form, bool askPassword, const wxString &pwd, bool
 
         if (database.IsEmpty())
         {
-            conn = new pgConn(GetName(), DEFAULT_PG_DATABASE, username, password, port, ssl, 0, _("pgAdmin - Browser"));
+            conn = new pgConn(GetName(), DEFAULT_PG_DATABASE, username, password, port, rolename, ssl, 0, _("pgAdmin - Browser"));
             if (conn->GetStatus() == PGCONN_OK)
                 database=DEFAULT_PG_DATABASE;
             else if (conn->GetStatus() == PGCONN_BAD && conn->GetLastError().Find(
                                 wxT("database \"") DEFAULT_PG_DATABASE wxT("\" does not exist")) >= 0)
             {
                 delete conn;
-                conn = new pgConn(GetName(), wxT("template1"), username, password, port, ssl, 0, _("pgAdmin - Browser"));
+                conn = new pgConn(GetName(), wxT("template1"), username, password, port, rolename, ssl, 0, _("pgAdmin - Browser"));
                 if (conn && conn->GetStatus() == PGCONN_OK)
                     database=wxT("template1");
             }
         }
         else
         {
-            conn = new pgConn(GetName(), database, username, password, port, ssl, 0, _("pgAdmin - Browser"));
+            conn = new pgConn(GetName(), database, username, password, port, rolename, ssl, 0, _("pgAdmin - Browser"));
             if (!conn)
             {
                 form->EndMsg(false);
@@ -1124,7 +1125,7 @@ pgObject *pgServerFactory::CreateObjects(pgCollection *obj, ctlTree *browser, co
     long numServers=settings->Read(wxT("Servers/Count"), 0L);
 
     long loop, port, ssl=0;
-    wxString key, servername, description, database, username, lastDatabase, lastSchema, storePwd, restore, serviceID, discoveryID, dbRestriction, colour, group;
+    wxString key, servername, description, database, username, lastDatabase, lastSchema, storePwd, rolename, restore, serviceID, discoveryID, dbRestriction, colour, group;
     pgServer *server=0;
 
     wxArrayString discoveredServers;
@@ -1146,6 +1147,7 @@ pgObject *pgServerFactory::CreateObjects(pgCollection *obj, ctlTree *browser, co
         settings->Read(key + wxT("DiscoveryID"), &discoveryID, serviceID);
         settings->Read(key + wxT("Description"), &description, wxEmptyString);
         settings->Read(key + wxT("StorePwd"), &storePwd, wxEmptyString);
+        settings->Read(key + wxT("Rolename"), &rolename, wxEmptyString);
         settings->Read(key + wxT("Restore"), &restore, wxT("true"));
         settings->Read(key + wxT("Port"), &port, 0);
         settings->Read(key + wxT("Database"), &database, wxEmptyString);
@@ -1189,7 +1191,7 @@ pgObject *pgServerFactory::CreateObjects(pgCollection *obj, ctlTree *browser, co
         }
 
         // Add the Server node
-        server = new pgServer(servername, description, database, username, port, StrToBool(storePwd), StrToBool(restore), ssl);
+        server = new pgServer(servername, description, database, username, port, StrToBool(storePwd), rolename, StrToBool(restore), ssl);
         server->iSetLastDatabase(lastDatabase);
         server->iSetLastSchema(lastSchema);
         server->iSetServiceID(serviceID);
@@ -1362,7 +1364,7 @@ pgObject *pgServerFactory::CreateObjects(pgCollection *obj, ctlTree *browser, co
                     // Add the item, if it looks sane
                     if (port != 0 && username != wxEmptyString)
                     {
-                        server = new pgServer(servername, description, wxT("postgres"), username, port, false, 0);
+                        server = new pgServer(servername, description, wxT("postgres"), username, port, false, rolename, 0);
                         server->iSetDiscoveryID(cnf->GetPath() + wxT("/") + version);
                         server->iSetDiscovered(true);
 						server->iSetGroup(group);
@@ -1415,7 +1417,7 @@ pgObject *pgServerFactory::CreateObjects(pgCollection *obj, ctlTree *browser, co
                     // Add the item, if it looks sane
                     if (port != 0 && username != wxEmptyString)
                     {
-                        server = new pgServer(servername, description, wxT("edb"), username, port, false, 0);
+                        server = new pgServer(servername, description, wxT("edb"), username, port, false, rolename, 0);
                         server->iSetDiscoveryID(cnf->GetPath() + wxT("/") + version);
                         server->iSetDiscovered(true);
                         browser->AppendItem(browser->GetFirstChild(obj->GetId(), groupcookie), server->GetFullName(), server->GetIconId(), -1, server);
diff --git a/pgadmin/ui/dlgSelectConnection.xrc b/pgadmin/ui/dlgSelectConnection.xrc
index 6d41c36..b19c602 100644
--- a/pgadmin/ui/dlgSelectConnection.xrc
+++ b/pgadmin/ui/dlgSelectConnection.xrc
@@ -20,6 +20,14 @@
       <pos>65,35d</pos>
       <size>135,-1d</size>
     </object>
+    <object class="wxStaticText" name="stRolename">
+      <label>Rolename</label>
+      <pos>5,37d</pos>
+    </object>
+    <object class="wxTextCtrl" name="txtRolename">
+      <pos>65,35d</pos>
+      <size>135,-1d</size>
+    </object>
     <object class="wxButton" name="wxID_HELP">
       <label>&amp;Help</label>
       <pos>2,65d</pos>
diff --git a/pgadmin/ui/dlgServer.xrc b/pgadmin/ui/dlgServer.xrc
index e6886ba..ab5b8bd 100644
--- a/pgadmin/ui/dlgServer.xrc
+++ b/pgadmin/ui/dlgServer.xrc
@@ -17,10 +17,10 @@
             <object class="wxPanel" name="pnlProperties">
               <object class="wxFlexGridSizer">
                 <cols>2</cols>
-                <rows>13</rows>
+                <rows>14</rows>
                 <vgap>5</vgap>
                 <hgap>5</hgap>
-                <growablerows>9</growablerows>
+                <growablerows>10</growablerows>
                 <growablecols>1</growablecols>
                 <object class="sizeritem">
                   <object class="wxStaticText" name="stDescription">
@@ -130,6 +130,18 @@
                   <border>4</border>
                 </object>
                 <object class="sizeritem">
+                  <object class="wxStaticText" name="stRolename">
+                    <label>Rolename</label>
+                  </object>
+                  <flag>wxALIGN_CENTRE_VERTICAL|wxTOP|wxLEFT|wxRIGHT</flag>
+                  <border>4</border>
+                </object>
+                <object class="sizeritem">
+                  <object class="wxTextCtrl" name="txtRolename"/>
+                  <flag>wxEXPAND|wxALIGN_CENTRE_VERTICAL|wxTOP|wxLEFT|wxRIGHT</flag>
+                  <border>4</border>
+                </object>
+                <object class="sizeritem">
                   <object class="wxStaticText" name="stRestore">
                     <label>Restore env?</label>
                   </object>
diff --git a/pgadmin/ui/xrcDialogs.cpp b/pgadmin/ui/xrcDialogs.cpp
index 22a146e..b910c6c 100644
--- a/pgadmin/ui/xrcDialogs.cpp
+++ b/pgadmin/ui/xrcDialogs.cpp
@@ -15917,7 +15917,7 @@ static unsigned char xml_res_file_42[] = {
 99,116,62,10,32,32,60,47,111,98,106,101,99,116,62,10,60,47,114,101,115,
 111,117,114,99,101,62,10};
 
-static size_t xml_res_size_43 = 1176;
+static size_t xml_res_size_43 = 1411;
 static unsigned char xml_res_file_43[] = {
 60,63,120,109,108,32,118,101,114,115,105,111,110,61,34,49,46,48,34,32,101,
 110,99,111,100,105,110,103,61,34,73,83,79,45,56,56,53,57,45,49,34,63,62,
@@ -15953,28 +15953,39 @@ static unsigned char xml_res_file_43[] = {
 32,32,32,60,112,111,115,62,54,53,44,51,53,100,60,47,112,111,115,62,10,32,
 32,32,32,32,32,60,115,105,122,101,62,49,51,53,44,45,49,100,60,47,115,105,
 122,101,62,10,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,
-60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,66,117,116,
-116,111,110,34,32,110,97,109,101,61,34,119,120,73,68,95,72,69,76,80,34,
-62,10,32,32,32,32,32,32,60,108,97,98,101,108,62,38,97,109,112,59,72,101,
-108,112,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,60,112,111,115,
-62,50,44,54,53,100,60,47,112,111,115,62,10,32,32,32,32,32,32,60,115,116,
-121,108,101,47,62,10,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,
-32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,66,117,
-116,116,111,110,34,32,110,97,109,101,61,34,119,120,73,68,95,79,75,34,62,
-10,32,32,32,32,32,32,60,108,97,98,101,108,62,38,97,109,112,59,79,75,60,
-47,108,97,98,101,108,62,10,32,32,32,32,32,32,60,100,101,102,97,117,108,
-116,62,49,60,47,100,101,102,97,117,108,116,62,10,32,32,32,32,32,32,60,112,
-111,115,62,57,55,44,54,53,100,60,47,112,111,115,62,10,32,32,32,32,32,32,
-60,115,116,121,108,101,47,62,10,32,32,32,32,60,47,111,98,106,101,99,116,
-62,10,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,
-120,66,117,116,116,111,110,34,32,110,97,109,101,61,34,119,120,73,68,95,
-67,65,78,67,69,76,34,62,10,32,32,32,32,32,32,60,108,97,98,101,108,62,38,
-97,109,112,59,67,97,110,99,101,108,60,47,108,97,98,101,108,62,10,32,32,
-32,32,32,32,60,100,101,102,97,117,108,116,62,48,60,47,100,101,102,97,117,
-108,116,62,10,32,32,32,32,32,32,60,112,111,115,62,49,53,48,44,54,53,100,
-60,47,112,111,115,62,10,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,
-32,60,47,111,98,106,101,99,116,62,10,60,47,114,101,115,111,117,114,99,101,
-62,10};
+60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,83,116,97,116,
+105,99,84,101,120,116,34,32,110,97,109,101,61,34,115,116,82,111,108,101,
+110,97,109,101,34,62,10,32,32,32,32,32,32,60,108,97,98,101,108,62,82,111,
+108,101,110,97,109,101,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,
+60,112,111,115,62,53,44,51,55,100,60,47,112,111,115,62,10,32,32,32,32,60,
+47,111,98,106,101,99,116,62,10,32,32,32,32,60,111,98,106,101,99,116,32,
+99,108,97,115,115,61,34,119,120,84,101,120,116,67,116,114,108,34,32,110,
+97,109,101,61,34,116,120,116,82,111,108,101,110,97,109,101,34,62,10,32,
+32,32,32,32,32,60,112,111,115,62,54,53,44,51,53,100,60,47,112,111,115,62,
+10,32,32,32,32,32,32,60,115,105,122,101,62,49,51,53,44,45,49,100,60,47,
+115,105,122,101,62,10,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,
+32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,66,
+117,116,116,111,110,34,32,110,97,109,101,61,34,119,120,73,68,95,72,69,76,
+80,34,62,10,32,32,32,32,32,32,60,108,97,98,101,108,62,38,97,109,112,59,
+72,101,108,112,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,60,112,111,
+115,62,50,44,54,53,100,60,47,112,111,115,62,10,32,32,32,32,32,32,60,115,
+116,121,108,101,47,62,10,32,32,32,32,60,47,111,98,106,101,99,116,62,10,
+32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,
+66,117,116,116,111,110,34,32,110,97,109,101,61,34,119,120,73,68,95,79,75,
+34,62,10,32,32,32,32,32,32,60,108,97,98,101,108,62,38,97,109,112,59,79,
+75,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,60,100,101,102,97,117,
+108,116,62,49,60,47,100,101,102,97,117,108,116,62,10,32,32,32,32,32,32,
+60,112,111,115,62,57,55,44,54,53,100,60,47,112,111,115,62,10,32,32,32,32,
+32,32,60,115,116,121,108,101,47,62,10,32,32,32,32,60,47,111,98,106,101,
+99,116,62,10,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,
+61,34,119,120,66,117,116,116,111,110,34,32,110,97,109,101,61,34,119,120,
+73,68,95,67,65,78,67,69,76,34,62,10,32,32,32,32,32,32,60,108,97,98,101,
+108,62,38,97,109,112,59,67,97,110,99,101,108,60,47,108,97,98,101,108,62,
+10,32,32,32,32,32,32,60,100,101,102,97,117,108,116,62,48,60,47,100,101,
+102,97,117,108,116,62,10,32,32,32,32,32,32,60,112,111,115,62,49,53,48,44,
+54,53,100,60,47,112,111,115,62,10,32,32,32,32,60,47,111,98,106,101,99,116,
+62,10,32,32,60,47,111,98,106,101,99,116,62,10,60,47,114,101,115,111,117,
+114,99,101,62,10};
 
 static size_t xml_res_size_44 = 9552;
 static unsigned char xml_res_file_44[] = {
@@ -16414,7 +16425,7 @@ static unsigned char xml_res_file_44[] = {
 99,116,62,10,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,60,47,
 111,98,106,101,99,116,62,10,60,47,114,101,115,111,117,114,99,101,62,10};
 
-static size_t xml_res_size_45 = 11187;
+static size_t xml_res_size_45 = 11765;
 static unsigned char xml_res_file_45[] = {
 60,63,120,109,108,32,118,101,114,115,105,111,110,61,34,49,46,48,34,32,101,
 110,99,111,100,105,110,103,61,34,73,83,79,45,56,56,53,57,45,49,34,63,62,
@@ -16452,21 +16463,47 @@ static unsigned char xml_res_file_45[] = {
 101,120,71,114,105,100,83,105,122,101,114,34,62,10,32,32,32,32,32,32,32,
 32,32,32,32,32,32,32,32,32,60,99,111,108,115,62,50,60,47,99,111,108,115,
 62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,114,111,119,115,
-62,49,51,60,47,114,111,119,115,62,10,32,32,32,32,32,32,32,32,32,32,32,32,
+62,49,52,60,47,114,111,119,115,62,10,32,32,32,32,32,32,32,32,32,32,32,32,
 32,32,32,32,60,118,103,97,112,62,53,60,47,118,103,97,112,62,10,32,32,32,
 32,32,32,32,32,32,32,32,32,32,32,32,32,60,104,103,97,112,62,53,60,47,104,
 103,97,112,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,103,
-114,111,119,97,98,108,101,114,111,119,115,62,57,60,47,103,114,111,119,97,
-98,108,101,114,111,119,115,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,60,103,114,111,119,97,98,108,101,99,111,108,115,62,49,60,47,103,
-114,111,119,97,98,108,101,99,111,108,115,62,10,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,
-34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,
-115,61,34,119,120,83,116,97,116,105,99,84,101,120,116,34,32,110,97,109,
-101,61,34,115,116,68,101,115,99,114,105,112,116,105,111,110,34,62,10,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,
-108,62,78,97,109,101,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,32,
+114,111,119,97,98,108,101,114,111,119,115,62,49,48,60,47,103,114,111,119,
+97,98,108,101,114,111,119,115,62,10,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,60,103,114,111,119,97,98,108,101,99,111,108,115,62,49,60,47,
+103,114,111,119,97,98,108,101,99,111,108,115,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,
+61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,
+115,115,61,34,119,120,83,116,97,116,105,99,84,101,120,116,34,32,110,97,
+109,101,61,34,115,116,68,101,115,99,114,105,112,116,105,111,110,34,62,10,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,
+101,108,62,78,97,109,101,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,
+119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,
+124,119,120,84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,72,84,
+60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,60,98,111,114,100,101,114,62,52,60,47,98,111,114,100,101,114,62,
+10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,
+99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,
+106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,
+109,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,
+98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,84,101,120,116,67,
+116,114,108,34,32,110,97,109,101,61,34,116,120,116,68,101,115,99,114,105,
+112,116,105,111,110,34,47,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,60,102,108,97,103,62,119,120,69,88,80,65,78,68,124,119,120,
+65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,124,119,
+120,84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,72,84,60,47,102,
+108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+60,98,111,114,100,101,114,62,52,60,47,98,111,114,100,101,114,62,10,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,
+10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,
+116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,109,34,62,
+10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,
+101,99,116,32,99,108,97,115,115,61,34,119,120,83,116,97,116,105,99,84,101,
+120,116,34,32,110,97,109,101,61,34,115,116,78,97,109,101,34,62,10,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,
+108,62,72,111,115,116,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,32,
 32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,
 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,
 120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,124,
@@ -16478,50 +16515,24 @@ static unsigned char xml_res_file_45[] = {
 101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,109,
 34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,
 106,101,99,116,32,99,108,97,115,115,61,34,119,120,84,101,120,116,67,116,
-114,108,34,32,110,97,109,101,61,34,116,120,116,68,101,115,99,114,105,112,
-116,105,111,110,34,47,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,60,102,108,97,103,62,119,120,69,88,80,65,78,68,124,119,120,65,
-76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,124,119,120,
-84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,72,84,60,47,102,108,
-97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,98,
-111,114,100,101,114,62,52,60,47,98,111,114,100,101,114,62,10,32,32,32,32,
-32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,
-99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,
-116,32,99,108,97,115,115,61,34,119,120,83,116,97,116,105,99,84,101,120,
-116,34,32,110,97,109,101,61,34,115,116,78,97,109,101,34,62,10,32,32,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,
-62,72,111,115,116,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,
-65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,124,119,
-120,84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,72,84,60,47,102,
-108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-60,98,111,114,100,101,114,62,52,60,47,98,111,114,100,101,114,62,10,32,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,
-10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,
-116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,109,34,62,
-10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,
-101,99,116,32,99,108,97,115,115,61,34,119,120,84,101,120,116,67,116,114,
-108,34,32,110,97,109,101,61,34,116,120,116,78,97,109,101,34,47,62,10,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,
-119,120,69,88,80,65,78,68,124,119,120,65,76,73,71,78,95,67,69,78,84,82,
-69,95,86,69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,120,76,69,70,
-84,124,119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,52,
-60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,
-115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,
-61,34,119,120,83,116,97,116,105,99,84,101,120,116,34,32,110,97,109,101,
-61,34,115,116,80,111,114,116,34,62,10,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,62,80,111,114,116,60,47,
-108,97,98,101,108,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,65,76,73,71,78,95,67,
-69,78,84,82,69,95,86,69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,
+114,108,34,32,110,97,109,101,61,34,116,120,116,78,97,109,101,34,47,62,10,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,
+62,119,120,69,88,80,65,78,68,124,119,120,65,76,73,71,78,95,67,69,78,84,
+82,69,95,86,69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,120,76,69,
+70,84,124,119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,
+52,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,
+34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,
+115,61,34,119,120,83,116,97,116,105,99,84,101,120,116,34,32,110,97,109,
+101,61,34,115,116,80,111,114,116,34,62,10,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,62,80,111,114,116,60,
+47,108,97,98,101,108,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,65,76,73,71,78,95,
+67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,
 120,76,69,70,84,124,119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,
 32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,
 114,62,52,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,
@@ -16690,95 +16701,23 @@ static unsigned char xml_res_file_45[] = {
 115,61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,
 32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,
 97,115,115,61,34,119,120,83,116,97,116,105,99,84,101,120,116,34,32,110,
-97,109,101,61,34,115,116,82,101,115,116,111,114,101,34,62,10,32,32,32,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,62,
-82,101,115,116,111,114,101,32,101,110,118,63,60,47,108,97,98,101,108,62,
-10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,
-101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-60,102,108,97,103,62,119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,
-69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,120,76,69,70,84,124,119,
-120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,52,60,47,98,111,
-114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,
-47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,
-114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,
-67,104,101,99,107,66,111,120,34,32,110,97,109,101,61,34,99,104,107,82,101,
-115,116,111,114,101,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,60,108,97,98,101,108,47,62,10,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,32,32,32,60,99,104,101,99,107,101,100,62,49,
-60,47,99,104,101,99,107,101,100,62,10,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,69,88,
-80,65,78,68,124,119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,
-84,73,67,65,76,124,119,120,84,79,80,124,119,120,76,69,70,84,124,119,120,
-82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,52,60,47,98,111,114,
-100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,
-111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,
-114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,
-83,116,97,116,105,99,84,101,120,116,34,32,110,97,109,101,61,34,115,116,
-68,98,82,101,115,116,114,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,60,108,97,98,101,108,62,68,66,32,114,101,115,116,114,
-105,99,116,105,111,110,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,
-119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,
-124,119,120,84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,72,84,
-60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,60,98,111,114,100,101,114,62,52,60,47,98,111,114,100,101,114,62,
-10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,
-99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,
-106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,
-109,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,
-98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,84,101,120,116,67,
-116,114,108,34,32,110,97,109,101,61,34,116,120,116,68,98,82,101,115,116,
-114,105,99,116,105,111,110,34,47,62,10,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,69,88,80,65,78,68,124,
-119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,
-124,119,120,84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,72,84,
-60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,60,98,111,114,100,101,114,62,52,60,47,98,111,114,100,101,114,62,
-10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,
-99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,
-106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,
-109,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,
-98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,83,116,97,116,105,
-99,84,101,120,116,34,32,110,97,109,101,61,34,115,116,83,101,114,118,105,
-99,101,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,60,108,97,98,101,108,62,83,101,114,118,105,99,101,60,47,108,97,98,101,
-108,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,
+97,109,101,61,34,115,116,82,111,108,101,110,97,109,101,34,62,10,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,
+62,82,111,108,101,110,97,109,101,60,47,108,97,98,101,108,62,10,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,
+62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,
+97,103,62,119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,
+67,65,76,124,119,120,84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,
+71,72,84,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,60,98,111,114,100,101,114,62,52,60,47,98,111,114,100,
+101,114,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,
 98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,60,102,108,97,103,62,119,120,65,76,73,71,78,95,67,69,78,84,82,69,
-95,86,69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,120,76,69,70,84,
-124,119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,52,60,
-47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,
-105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,
-34,119,120,84,101,120,116,67,116,114,108,34,32,110,97,109,101,61,34,116,
-120,116,83,101,114,118,105,99,101,34,47,62,10,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,69,88,80,65,78,
-68,124,119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,
-65,76,124,119,120,84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,
-72,84,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,60,98,111,114,100,101,114,62,52,60,47,98,111,114,100,101,
-114,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,
-106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,
-111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,
-116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,83,116,97,
-116,105,99,84,101,120,116,34,32,110,97,109,101,61,34,115,116,84,114,121,
-67,111,110,110,101,99,116,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,60,108,97,98,101,108,62,67,111,110,110,101,99,116,
-32,110,111,119,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,
+60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,114,
+105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,84,101,
+120,116,67,116,114,108,34,32,110,97,109,101,61,34,116,120,116,82,111,108,
+101,110,97,109,101,34,47,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,60,102,108,97,103,62,119,120,69,88,80,65,78,68,124,119,120,
 65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,124,119,
 120,84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,72,84,60,47,102,
 108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
@@ -16787,145 +16726,243 @@ static unsigned char xml_res_file_45[] = {
 10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,
 116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,109,34,62,
 10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,
-101,99,116,32,99,108,97,115,115,61,34,119,120,67,104,101,99,107,66,111,
-120,34,32,110,97,109,101,61,34,99,104,107,84,114,121,67,111,110,110,101,
-99,116,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,60,108,97,98,101,108,47,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,60,99,104,101,99,107,101,100,62,49,60,47,99,104,101,
-99,107,101,100,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+101,99,116,32,99,108,97,115,115,61,34,119,120,83,116,97,116,105,99,84,101,
+120,116,34,32,110,97,109,101,61,34,115,116,82,101,115,116,111,114,101,34,
+62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,
+97,98,101,108,62,82,101,115,116,111,114,101,32,101,110,118,63,60,47,108,
+97,98,101,108,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
 32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,60,102,108,97,103,62,119,120,69,88,80,65,78,68,124,119,
-120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,124,
-119,120,84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,72,84,60,
-47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,60,98,111,114,100,101,114,62,52,60,47,98,111,114,100,101,114,62,10,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,
-116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,
-101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,109,
-34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,
-106,101,99,116,32,99,108,97,115,115,61,34,119,120,83,116,97,116,105,99,
-84,101,120,116,34,32,110,97,109,101,61,34,115,116,67,111,108,111,117,114,
-34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,
-108,97,98,101,108,62,67,111,108,111,117,114,60,47,108,97,98,101,108,62,
-10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,
-101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-60,102,108,97,103,62,119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,
-69,82,84,73,67,65,76,124,119,120,65,76,76,60,47,102,108,97,103,62,10,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,
-114,62,52,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,
-115,61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,
-97,115,115,61,34,99,116,108,67,111,108,111,117,114,80,105,99,107,101,114,
-34,32,110,97,109,101,61,34,99,111,108,111,117,114,80,105,99,107,101,114,
-34,47,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,
-108,97,103,62,119,120,69,88,80,65,78,68,124,119,120,65,76,73,71,78,95,67,
-69,78,84,82,69,95,86,69,82,84,73,67,65,76,124,119,120,65,76,76,60,47,102,
+32,32,32,32,32,32,60,102,108,97,103,62,119,120,65,76,73,71,78,95,67,69,
+78,84,82,69,95,86,69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,120,
+76,69,70,84,124,119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,
+62,52,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,
+61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,
+115,115,61,34,119,120,67,104,101,99,107,66,111,120,34,32,110,97,109,101,
+61,34,99,104,107,82,101,115,116,111,114,101,34,62,10,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,47,62,10,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,99,104,101,
+99,107,101,100,62,49,60,47,99,104,101,99,107,101,100,62,10,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,
+10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,
+103,62,119,120,69,88,80,65,78,68,124,119,120,65,76,73,71,78,95,67,69,78,
+84,82,69,95,86,69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,120,76,
+69,70,84,124,119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,
+62,52,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,
+61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,
+115,115,61,34,119,120,83,116,97,116,105,99,84,101,120,116,34,32,110,97,
+109,101,61,34,115,116,68,98,82,101,115,116,114,34,62,10,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,62,68,
+66,32,114,101,115,116,114,105,99,116,105,111,110,60,47,108,97,98,101,108,
+62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,
+106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,60,102,108,97,103,62,119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,
+86,69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,120,76,69,70,84,124,
+119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,52,60,47,
+98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,
+105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,
+34,119,120,84,101,120,116,67,116,114,108,34,32,110,97,109,101,61,34,116,
+120,116,68,98,82,101,115,116,114,105,99,116,105,111,110,34,47,62,10,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,
+119,120,69,88,80,65,78,68,124,119,120,65,76,73,71,78,95,67,69,78,84,82,
+69,95,86,69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,120,76,69,70,
+84,124,119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,52,
+60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,
+115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,
+61,34,119,120,83,116,97,116,105,99,84,101,120,116,34,32,110,97,109,101,
+61,34,115,116,83,101,114,118,105,99,101,34,62,10,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,62,83,101,114,
+118,105,99,101,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,
+65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,124,119,
+120,84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,72,84,60,47,102,
 108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
 60,98,111,114,100,101,114,62,52,60,47,98,111,114,100,101,114,62,10,32,32,
 32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,
 10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,
 116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,109,34,62,
 10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,
-101,99,116,32,99,108,97,115,115,61,34,119,120,83,116,97,116,105,99,84,101,
-120,116,34,32,110,97,109,101,61,34,115,116,71,114,111,117,112,34,62,10,
-32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,
-101,108,62,71,114,111,117,112,60,47,108,97,98,101,108,62,10,32,32,32,32,
+101,99,116,32,99,108,97,115,115,61,34,119,120,84,101,120,116,67,116,114,
+108,34,32,110,97,109,101,61,34,116,120,116,83,101,114,118,105,99,101,34,
+47,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,
+97,103,62,119,120,69,88,80,65,78,68,124,119,120,65,76,73,71,78,95,67,69,
+78,84,82,69,95,86,69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,120,
+76,69,70,84,124,119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,
+62,52,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,
+61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,
+115,115,61,34,119,120,83,116,97,116,105,99,84,101,120,116,34,32,110,97,
+109,101,61,34,115,116,84,114,121,67,111,110,110,101,99,116,34,62,10,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,
+108,62,67,111,110,110,101,99,116,32,110,111,119,60,47,108,97,98,101,108,
+62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,
+106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,60,102,108,97,103,62,119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,
+86,69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,120,76,69,70,84,124,
+119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,52,60,47,
+98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,
+105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,
+34,119,120,67,104,101,99,107,66,111,120,34,32,110,97,109,101,61,34,99,104,
+107,84,114,121,67,111,110,110,101,99,116,34,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,47,62,10,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,99,104,101,
+99,107,101,100,62,49,60,47,99,104,101,99,107,101,100,62,10,32,32,32,32,
 32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,
 10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,
-103,62,119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,
-65,76,124,119,120,84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,
-72,84,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,60,98,111,114,100,101,114,62,52,60,47,98,111,114,100,101,
-114,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,
-106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,
-111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,
-116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,67,111,109,
-98,111,66,111,120,34,32,110,97,109,101,61,34,99,98,71,114,111,117,112,34,
-62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,115,
-116,121,108,101,62,119,120,67,66,95,68,82,79,80,68,79,87,78,60,47,115,116,
-121,108,101,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-32,32,60,99,111,110,116,101,110,116,47,62,10,32,32,32,32,32,32,32,32,32,
+103,62,119,120,69,88,80,65,78,68,124,119,120,65,76,73,71,78,95,67,69,78,
+84,82,69,95,86,69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,120,76,
+69,70,84,124,119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,
+62,52,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,
+61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,
+115,115,61,34,119,120,83,116,97,116,105,99,84,101,120,116,34,32,110,97,
+109,101,61,34,115,116,67,111,108,111,117,114,34,62,10,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,62,67,111,
+108,111,117,114,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,32,32,32,
 32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,
 32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,
-69,88,80,65,78,68,124,119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,
-69,82,84,73,67,65,76,124,119,120,84,79,80,124,119,120,76,69,70,84,124,119,
-120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,52,60,47,98,111,
-114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,
-47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,
-47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,60,47,111,
-98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,
-116,62,10,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,69,88,80,
-65,78,68,124,119,120,65,76,73,71,78,95,67,69,78,84,82,69,124,119,120,65,
-76,76,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,60,98,111,114,
-100,101,114,62,51,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,
-60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,60,111,98,106,101,99,
-116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,109,34,62,
-10,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,
-61,34,119,120,70,108,101,120,71,114,105,100,83,105,122,101,114,34,62,10,
-32,32,32,32,32,32,32,32,32,32,60,99,111,108,115,62,52,60,47,99,111,108,
-115,62,10,32,32,32,32,32,32,32,32,32,32,60,103,114,111,119,97,98,108,101,
-99,111,108,115,62,49,60,47,103,114,111,119,97,98,108,101,99,111,108,115,
-62,10,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,
-97,115,115,61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,
+65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,124,119,
+120,65,76,76,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,52,60,47,98,111,114,100,
+101,114,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,
+98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,114,
+105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,99,116,108,67,
+111,108,111,117,114,80,105,99,107,101,114,34,32,110,97,109,101,61,34,99,
+111,108,111,117,114,80,105,99,107,101,114,34,47,62,10,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,69,88,
+80,65,78,68,124,119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,
+84,73,67,65,76,124,119,120,65,76,76,60,47,102,108,97,103,62,10,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,
+62,52,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,
 32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,
-61,34,119,120,66,117,116,116,111,110,34,32,110,97,109,101,61,34,119,120,
-73,68,95,72,69,76,80,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
-60,108,97,98,101,108,62,72,101,108,112,60,47,108,97,98,101,108,62,10,32,
-32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,
-32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,69,88,80,65,
-78,68,124,119,120,65,76,76,60,47,102,108,97,103,62,10,32,32,32,32,32,32,
-32,32,32,32,32,32,60,98,111,114,100,101,114,62,51,60,47,98,111,114,100,
-101,114,62,10,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,
-62,10,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,
-97,115,115,61,34,115,112,97,99,101,114,34,62,10,32,32,32,32,32,32,32,32,
-32,32,32,32,60,115,105,122,101,62,48,44,48,100,60,47,115,105,122,101,62,
-10,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,
-32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,
-34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,
-32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,
-66,117,116,116,111,110,34,32,110,97,109,101,61,34,119,120,73,68,95,79,75,
+61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,
+115,115,61,34,119,120,83,116,97,116,105,99,84,101,120,116,34,32,110,97,
+109,101,61,34,115,116,71,114,111,117,112,34,62,10,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,62,71,114,111,
+117,112,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,65,76,
+73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,124,119,120,84,
+79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,72,84,60,47,102,108,
+97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,98,
+111,114,100,101,114,62,52,60,47,98,111,114,100,101,114,62,10,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,
+99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,
+116,32,99,108,97,115,115,61,34,119,120,67,111,109,98,111,66,111,120,34,
+32,110,97,109,101,61,34,99,98,71,114,111,117,112,34,62,10,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,115,116,121,108,101,62,
+119,120,67,66,95,68,82,79,80,68,79,87,78,60,47,115,116,121,108,101,62,10,
+32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,99,111,110,
+116,101,110,116,47,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,69,88,80,65,78,68,124,
+119,120,65,76,73,71,78,95,67,69,78,84,82,69,95,86,69,82,84,73,67,65,76,
+124,119,120,84,79,80,124,119,120,76,69,70,84,124,119,120,82,73,71,72,84,
+60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,60,98,111,114,100,101,114,62,52,60,47,98,111,114,100,101,114,62,
+10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,
+99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,
+101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,
+99,116,62,10,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,
+62,10,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,
+32,32,32,32,32,60,102,108,97,103,62,119,120,69,88,80,65,78,68,124,119,120,
+65,76,73,71,78,95,67,69,78,84,82,69,124,119,120,65,76,76,60,47,102,108,
+97,103,62,10,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,51,60,
+47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,60,47,111,98,106,101,
+99,116,62,10,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,
+115,61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,
+32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,70,108,
+101,120,71,114,105,100,83,105,122,101,114,34,62,10,32,32,32,32,32,32,32,
+32,32,32,60,99,111,108,115,62,52,60,47,99,111,108,115,62,10,32,32,32,32,
+32,32,32,32,32,32,60,103,114,111,119,97,98,108,101,99,111,108,115,62,49,
+60,47,103,114,111,119,97,98,108,101,99,111,108,115,62,10,32,32,32,32,32,
+32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,
+105,122,101,114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,32,
+32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,66,117,
+116,116,111,110,34,32,110,97,109,101,61,34,119,120,73,68,95,72,69,76,80,
 34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,
-62,38,97,109,112,59,79,75,60,47,108,97,98,101,108,62,10,32,32,32,32,32,
-32,32,32,32,32,32,32,32,32,60,100,101,102,97,117,108,116,62,49,60,47,100,
-101,102,97,117,108,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,47,
-111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,102,
-108,97,103,62,119,120,69,88,80,65,78,68,124,119,120,65,76,76,60,47,102,
-108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,
-101,114,62,51,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,
+62,72,101,108,112,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,32,32,
+32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,
+32,32,32,60,102,108,97,103,62,119,120,69,88,80,65,78,68,124,119,120,65,
+76,76,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,
+98,111,114,100,101,114,62,51,60,47,98,111,114,100,101,114,62,10,32,32,32,
+32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,
+32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,112,
+97,99,101,114,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,115,105,122,
+101,62,48,44,48,100,60,47,115,105,122,101,62,10,32,32,32,32,32,32,32,32,
 32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,60,
 111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,
 116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,
 101,99,116,32,99,108,97,115,115,61,34,119,120,66,117,116,116,111,110,34,
-32,110,97,109,101,61,34,119,120,73,68,95,67,65,78,67,69,76,34,62,10,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,62,38,97,109,
-112,59,67,97,110,99,101,108,60,47,108,97,98,101,108,62,10,32,32,32,32,32,
-32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,
-32,32,32,32,32,32,60,102,108,97,103,62,119,120,69,88,80,65,78,68,124,119,
-120,65,76,76,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,
-32,60,98,111,114,100,101,114,62,51,60,47,98,111,114,100,101,114,62,10,32,
-32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,
-32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,60,
-102,108,97,103,62,119,120,69,88,80,65,78,68,124,119,120,84,79,80,124,119,
-120,76,69,70,84,124,119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,
-32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,60,111,
-98,106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,114,105,116,
-101,109,34,62,10,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,
-108,97,115,115,61,34,119,120,83,116,97,116,117,115,66,97,114,34,32,110,
-97,109,101,61,34,117,110,107,83,116,97,116,117,115,66,97,114,34,62,10,32,
-32,32,32,32,32,32,32,32,32,60,115,116,121,108,101,62,119,120,83,84,95,83,
-73,90,69,71,82,73,80,60,47,115,116,121,108,101,62,10,32,32,32,32,32,32,
-32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,60,102,
-108,97,103,62,119,120,69,88,80,65,78,68,124,119,120,65,76,73,71,78,95,67,
-69,78,84,82,69,60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,60,98,
-111,114,100,101,114,62,51,60,47,98,111,114,100,101,114,62,10,32,32,32,32,
-32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,60,47,111,98,106,101,
-99,116,62,10,32,32,60,47,111,98,106,101,99,116,62,10,60,47,114,101,115,
-111,117,114,99,101,62,10};
+32,110,97,109,101,61,34,119,120,73,68,95,79,75,34,62,10,32,32,32,32,32,
+32,32,32,32,32,32,32,32,32,60,108,97,98,101,108,62,38,97,109,112,59,79,
+75,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,
+32,60,100,101,102,97,117,108,116,62,49,60,47,100,101,102,97,117,108,116,
+62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,99,116,62,
+10,32,32,32,32,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,69,
+88,80,65,78,68,124,119,120,65,76,76,60,47,102,108,97,103,62,10,32,32,32,
+32,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,51,60,47,98,111,
+114,100,101,114,62,10,32,32,32,32,32,32,32,32,32,32,60,47,111,98,106,101,
+99,116,62,10,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,
+99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,109,34,62,10,32,
+32,32,32,32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,
+115,115,61,34,119,120,66,117,116,116,111,110,34,32,110,97,109,101,61,34,
+119,120,73,68,95,67,65,78,67,69,76,34,62,10,32,32,32,32,32,32,32,32,32,
+32,32,32,32,32,60,108,97,98,101,108,62,38,97,109,112,59,67,97,110,99,101,
+108,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,
+47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,102,
+108,97,103,62,119,120,69,88,80,65,78,68,124,119,120,65,76,76,60,47,102,
+108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,100,
+101,114,62,51,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,32,32,
+32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,60,47,111,
+98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,
+120,69,88,80,65,78,68,124,119,120,84,79,80,124,119,120,76,69,70,84,124,
+119,120,82,73,71,72,84,60,47,102,108,97,103,62,10,32,32,32,32,32,32,60,
+47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,60,111,98,106,101,99,116,
+32,99,108,97,115,115,61,34,115,105,122,101,114,105,116,101,109,34,62,10,
+32,32,32,32,32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,
+34,119,120,83,116,97,116,117,115,66,97,114,34,32,110,97,109,101,61,34,117,
+110,107,83,116,97,116,117,115,66,97,114,34,62,10,32,32,32,32,32,32,32,32,
+32,32,60,115,116,121,108,101,62,119,120,83,84,95,83,73,90,69,71,82,73,80,
+60,47,115,116,121,108,101,62,10,32,32,32,32,32,32,32,32,60,47,111,98,106,
+101,99,116,62,10,32,32,32,32,32,32,32,32,60,102,108,97,103,62,119,120,69,
+88,80,65,78,68,124,119,120,65,76,73,71,78,95,67,69,78,84,82,69,60,47,102,
+108,97,103,62,10,32,32,32,32,32,32,32,32,60,98,111,114,100,101,114,62,51,
+60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,60,47,111,98,106,101,
+99,116,62,10,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,60,47,
+111,98,106,101,99,116,62,10,60,47,114,101,115,111,117,114,99,101,62,10};
 
 static size_t xml_res_size_46 = 11122;
 static unsigned char xml_res_file_46[] = {
@@ -28870,7 +28907,7 @@ static unsigned char xml_res_file_67[] = {
 32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,60,47,111,98,106,101,
 99,116,62,10,60,47,114,101,115,111,117,114,99,101,62,10};
 
-static size_t xml_res_size_68 = 13600;
+static size_t xml_res_size_68 = 13700;
 static unsigned char xml_res_file_68[] = {
 60,63,120,109,108,32,118,101,114,115,105,111,110,61,34,49,46,48,34,32,101,
 110,99,111,100,105,110,103,61,34,73,83,79,45,56,56,53,57,45,49,34,63,62,
@@ -29451,15 +29488,20 @@ static unsigned char xml_res_file_68[] = {
 32,32,32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,
 66,117,116,116,111,110,34,32,110,97,109,101,61,34,98,116,110,86,105,101,
 119,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,60,108,97,98,101,
-108,62,38,97,109,112,59,86,105,101,119,60,47,108,97,98,101,108,62,10,32,
-32,32,32,32,32,32,32,32,32,32,32,32,32,60,100,101,102,97,117,108,116,62,
-49,60,47,100,101,102,97,117,108,116,62,10,32,32,32,32,32,32,32,32,32,32,
-32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,
-32,60,102,108,97,103,62,119,120,69,88,80,65,78,68,124,119,120,65,76,76,
-60,47,102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,
-114,100,101,114,62,51,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,
-32,32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,
-32,32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,
+108,62,38,97,109,112,59,68,105,115,112,108,97,121,32,111,98,106,101,99,
+116,115,60,47,108,97,98,101,108,62,10,32,32,32,32,32,32,32,32,32,32,32,
+32,32,32,60,116,111,111,108,116,105,112,62,68,105,115,112,108,97,121,32,
+111,98,106,101,99,116,115,32,99,111,110,116,97,105,110,101,100,32,105,110,
+32,116,104,101,32,100,117,109,112,32,111,110,32,97,32,115,112,101,99,105,
+102,105,99,32,116,97,98,60,47,116,111,111,108,116,105,112,62,10,32,32,32,
+32,32,32,32,32,32,32,32,32,32,32,60,100,101,102,97,117,108,116,62,49,60,
+47,100,101,102,97,117,108,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,
+60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,
+102,108,97,103,62,119,120,69,88,80,65,78,68,124,119,120,65,76,76,60,47,
+102,108,97,103,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,98,111,114,
+100,101,114,62,51,60,47,98,111,114,100,101,114,62,10,32,32,32,32,32,32,
+32,32,32,32,60,47,111,98,106,101,99,116,62,10,32,32,32,32,32,32,32,32,32,
+32,60,111,98,106,101,99,116,32,99,108,97,115,115,61,34,115,105,122,101,
 114,105,116,101,109,34,62,10,32,32,32,32,32,32,32,32,32,32,32,32,60,111,
 98,106,101,99,116,32,99,108,97,115,115,61,34,119,120,66,117,116,116,111,
 110,34,32,110,97,109,101,61,34,119,120,73,68,95,79,75,34,62,10,32,32,32,
diff --git a/pgadmin/utils/sysSettings.cpp b/pgadmin/utils/sysSettings.cpp
index 232d249..dca9477 100644
--- a/pgadmin/utils/sysSettings.cpp
+++ b/pgadmin/utils/sysSettings.cpp
@@ -52,6 +52,7 @@ sysSettings::sysSettings(const wxString& name) : wxConfig(name)
             moveStringValue(wxT("Servers/Server%d"), wxT("Servers/%d/Server"), i);
             moveStringValue(wxT("Servers/ServiceId%d"), wxT("Servers/%d/ServiceId"), i);
             moveStringValue(wxT("Servers/StorePWD%d"), wxT("Servers/%d/StorePWD"), i);
+            moveStringValue(wxT("Servers/Rolename%d"), wxT("Servers/%d/Rolename"), i);
             moveStringValue(wxT("Servers/Username%d"), wxT("Servers/%d/Username"), i);
             moveLongValue(wxT("Servers/Port%d"), wxT("Servers/%d/Port"), i);
             moveLongValue(wxT("Servers/SSL%d"), wxT("Servers/%d/SSL"), i);
-- 
1.7.1

